@@ -1,10 +1,10 @@
+from django.core.exceptions import ImproperlyConfigured
 from django.core.paginator import Page, Paginator
 from django.db import models
-from django.forms import BaseForm, ModelForm
+from django.forms import fields, BaseForm, Form, ModelForm
 from django.http import Http404
 from django.test import RequestFactory, TestCase
-from model_mommy import mommy
-from vanilla import CreateView, DetailView, DeleteView, ListView, UpdateView, View
+from vanilla import *
 import types
 
 
@@ -15,9 +15,13 @@ class Example(models.Model):
         ordering = ('id',)
 
 
+class ExampleForm(Form):
+    text = fields.CharField(max_length=10)
+
+
 class InstanceOf(object):
     """
-    We use this sential object together with our 'assertContext' helper method.
+    We use this sentinal object together with our 'assertContext' helper method.
 
     Used to ensure that a particular context value is an object of the given
     type, without requiring a specific fixed value.  Useful for form context,
@@ -27,6 +31,12 @@ class InstanceOf(object):
         self.expected_type = expected_type
 
 
+def create_instance(text=None, quantity=1):
+    for idx in range(quantity):
+        text = text or ('example %d' % idx)
+        Example.objects.create(text=text)
+
+
 class BaseTestCase(TestCase):
     def setUp(self):
         self.factory = RequestFactory()
@@ -77,7 +87,7 @@ class BaseTestCase(TestCase):
 
 class TestDetail(BaseTestCase):
     def test_detail(self):
-        mommy.make(Example, _quantity=3)
+        create_instance(quantity=3)
         pk = Example.objects.all()[0].pk
         view = DetailView.as_view(model=Example)
         response = self.get(view, pk=pk)
@@ -91,14 +101,52 @@ class TestDetail(BaseTestCase):
         })
 
     def test_detail_not_found(self):
-        mommy.make(Example, _quantity=3)
+        create_instance(quantity=3)
         view = DetailView.as_view(model=Example)
         self.assertRaises(Http404, self.get, view, pk=999)
 
+    def test_detail_misconfigured_urlconf(self):
+        # If we don't provide 'pk' in the URL conf,
+        # we should expect an ImproperlyConfigured exception.
+        create_instance(quantity=3)
+        view = DetailView.as_view(model=Example)
+        self.assertRaises(ImproperlyConfigured, self.get, view, slug=999)
+
+    def test_detail_misconfigured_template_name(self):
+        # If don't provide 'model' or 'template_name',
+        # we should expect an ImproperlyConfigured exception.
+        create_instance(quantity=3)
+        pk = Example.objects.all()[0].pk
+        view = DetailView.as_view(queryset=Example.objects.all())
+        self.assertRaises(ImproperlyConfigured, self.get, view, pk=pk)
+
+    def test_detail_misconfigured_queryset(self):
+        # If don't provide 'model' or 'queryset',
+        # we should expect an ImproperlyConfigured exception.
+        create_instance(quantity=3)
+        pk = Example.objects.all()[0].pk
+        view = DetailView.as_view(template_name='example.html')
+        self.assertRaises(ImproperlyConfigured, self.get, view, pk=pk)
+
+    def test_detail_missing_context_object_name(self):
+        # If don't provide 'model' or 'context_object_name',
+        # then the context will only contain the 'object' key.
+        create_instance(quantity=3)
+        pk = Example.objects.all()[0].pk
+        view = DetailView.as_view(queryset=Example.objects.all(), template_name='example.html')
+        response = self.get(view, pk=pk)
+
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.template_name, ['example.html'])
+        self.assertContext(response, {
+            'object': Example.objects.get(pk=pk),
+            'view': InstanceOf(View)
+        })
+
 
 class TestList(BaseTestCase):
     def test_list(self):
-        mommy.make(Example, _quantity=3)
+        create_instance(quantity=3)
         view = ListView.as_view(model=Example)
         response = self.get(view)
 
@@ -133,7 +181,7 @@ class TestList(BaseTestCase):
         self.assertRaises(Http404, self.get, view, pk=999)
 
     def test_paginated_list(self):
-        mommy.make(Example, _quantity=25)
+        create_instance(quantity=30)
         view = ListView.as_view(model=Example, paginate_by=10)
         response = self.get(view)
 
@@ -149,7 +197,7 @@ class TestList(BaseTestCase):
         })
 
     def test_paginated_list_valid_page_specified(self):
-        mommy.make(Example, _quantity=25)
+        create_instance(quantity=30)
         view = ListView.as_view(model=Example, paginate_by=10)
         response = self.get(view, page=2)
 
@@ -165,7 +213,7 @@ class TestList(BaseTestCase):
         })
 
     def test_paginated_list_last_page_specified(self):
-        mommy.make(Example, _quantity=25)
+        create_instance(quantity=30)
         view = ListView.as_view(model=Example, paginate_by=10)
         response = self.get(view, page='last')
 
@@ -181,12 +229,12 @@ class TestList(BaseTestCase):
         })
 
     def test_paginated_list_invalid_page_specified(self):
-        mommy.make(Example, _quantity=25)
+        create_instance(quantity=30)
         view = ListView.as_view(model=Example, paginate_by=10)
         self.assertRaises(Http404, self.get, view, page=999)
 
     def test_paginated_list_non_integer_page_specified(self):
-        mommy.make(Example, _quantity=25)
+        create_instance(quantity=30)
         view = ListView.as_view(model=Example, paginate_by=10)
         self.assertRaises(Http404, self.get, view, page='null')
 
@@ -226,10 +274,24 @@ class TestCreate(BaseTestCase):
         })
         self.assertFalse(Example.objects.exists())
 
+    def test_update_no_success_url(self):
+        view = CreateView.as_view(model=Example)
+        self.assertRaises(ImproperlyConfigured, self.post, view, data={'text': 'example'})
+
+    def test_detail_misconfigured_form_class(self):
+        # If don't provide 'model' or 'form_class',
+        # we should expect an ImproperlyConfigured exception.
+        view = CreateView.as_view(
+            queryset=Example.objects.all(),
+            template_name='example.html',
+            success_url='/success/'
+        )
+        self.assertRaises(ImproperlyConfigured, self.post, view, data={'text': 'example'})
+
 
 class TestUpdate(BaseTestCase):
     def test_update(self):
-        mommy.make(Example, _quantity=3)
+        create_instance(quantity=3)
         pk = Example.objects.all()[0].pk
         view = UpdateView.as_view(model=Example, success_url='/success/')
         response = self.post(view, pk=pk, data={'text': 'example'})
@@ -240,7 +302,7 @@ class TestUpdate(BaseTestCase):
         self.assertEqual(Example.objects.get(pk=pk).text, 'example')
 
     def test_update_failed(self):
-        mommy.make(Example, _quantity=3)
+        create_instance(quantity=3)
         pk = Example.objects.all()[0].pk
         original_text = Example.objects.all()[0].text
         view = UpdateView.as_view(model=Example, success_url='/success/')
@@ -259,7 +321,7 @@ class TestUpdate(BaseTestCase):
         self.assertEqual(Example.objects.get(pk=pk).text, original_text)
 
     def test_update_preview(self):
-        mommy.make(Example, _quantity=3)
+        create_instance(quantity=3)
         pk = Example.objects.all()[0].pk
         view = UpdateView.as_view(model=Example, success_url='/success/')
         response = self.get(view, pk=pk)
@@ -274,10 +336,16 @@ class TestUpdate(BaseTestCase):
         })
         self.assertEqual(Example.objects.count(), 3)
 
+    def test_update_no_success_url(self):
+        create_instance(quantity=3)
+        pk = Example.objects.all()[0].pk
+        view = UpdateView.as_view(model=Example)
+        self.assertRaises(ImproperlyConfigured, self.post, view, pk=pk, data={'text': 'example'})
+
 
 class TestDelete(BaseTestCase):
     def test_delete(self):
-        mommy.make(Example, _quantity=3)
+        create_instance(quantity=3)
         pk = Example.objects.all()[0].pk
         view = DeleteView.as_view(model=Example, success_url='/success/')
         response = self.post(view, pk=pk)
@@ -288,12 +356,12 @@ class TestDelete(BaseTestCase):
         self.assertFalse(Example.objects.filter(pk=pk).exists())
 
     def test_delete_not_found(self):
-        mommy.make(Example, _quantity=3)
+        create_instance(quantity=3)
         view = DeleteView.as_view(model=Example, success_url='/success/')
         self.assertRaises(Http404, self.get, view, pk=999)
 
     def test_delete_preview(self):
-        mommy.make(Example, _quantity=3)
+        create_instance(quantity=3)
         pk = Example.objects.all()[0].pk
         view = DeleteView.as_view(model=Example, success_url='/success/')
         response = self.get(view, pk=pk)
@@ -307,10 +375,16 @@ class TestDelete(BaseTestCase):
         })
         self.assertEqual(Example.objects.count(), 3)
 
+    def test_delete_no_success_url(self):
+        create_instance(quantity=3)
+        pk = Example.objects.all()[0].pk
+        view = DeleteView.as_view(model=Example)
+        self.assertRaises(ImproperlyConfigured, self.post, view, pk=pk)
+
 
 class TestAttributeOverrides(BaseTestCase):
     def test_template_name_override(self):
-        mommy.make(Example, _quantity=3)
+        create_instance(quantity=3)
         pk = Example.objects.all()[0].pk
         view = DetailView.as_view(model=Example, template_name='example.html')
         response = self.get(view, pk=pk)
@@ -324,7 +398,7 @@ class TestAttributeOverrides(BaseTestCase):
         })
 
     def test_template_name_suffix_override(self):
-        mommy.make(Example, _quantity=3)
+        create_instance(quantity=3)
         pk = Example.objects.all()[0].pk
         view = DetailView.as_view(model=Example, template_name_suffix='_suffix')
         response = self.get(view, pk=pk)
@@ -338,7 +412,7 @@ class TestAttributeOverrides(BaseTestCase):
         })
 
     def test_context_object_name_override(self):
-        mommy.make(Example, _quantity=3)
+        create_instance(quantity=3)
         pk = Example.objects.all()[0].pk
         view = DetailView.as_view(model=Example, context_object_name='current')
         response = self.get(view, pk=pk)
@@ -367,8 +441,8 @@ class TestAttributeOverrides(BaseTestCase):
         self.assertFalse(Example.objects.exists())
 
     def test_queryset_override(self):
-        mommy.make(Example, text='abc', _quantity=3)
-        mommy.make(Example, text='def', _quantity=3)
+        create_instance(text='abc', quantity=3)
+        create_instance(text='def', quantity=3)
         view = ListView.as_view(model=Example, queryset=Example.objects.filter(text='abc'))
         response = self.get(view)
 
@@ -381,4 +455,81 @@ class TestAttributeOverrides(BaseTestCase):
             'page_obj': None,
             'paginator': None,
             'is_paginated': False
-        })
\ No newline at end of file
+        })
+
+
+class TestTemplateView(BaseTestCase):
+    def test_template_view(self):
+        view = TemplateView.as_view(template_name='example.html')
+        response = self.get(view)
+
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.template_name, ['example.html'])
+        self.assertContext(response, {
+            'view': InstanceOf(View)
+        })
+
+    def test_misconfigured_template_view(self):
+        # A template view with no `template_name` is improperly configured.
+        view = TemplateView.as_view()
+        self.assertRaises(ImproperlyConfigured, self.get, view)
+
+
+class TestFormView(BaseTestCase):
+    def test_form_success(self):
+        view = FormView.as_view(
+            form_class=ExampleForm,
+            success_url='/success/',
+            template_name='example.html'
+        )
+        response = self.post(view, data={'text': 'example'})
+
+        self.assertEqual(response.status_code, 302)
+        self.assertEqual(response['location'], '/success/')
+
+    def test_form_failure(self):
+        view = FormView.as_view(
+            form_class=ExampleForm,
+            success_url='/success/',
+            template_name='example.html'
+        )
+        response = self.post(view, data={'text': 'example' * 100})
+
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.template_name, ['example.html'])
+        self.assertFormError(response, 'form', 'text', ['Ensure this value has at most 10 characters (it has 700).'])
+        self.assertContext(response, {
+            'form': InstanceOf(BaseForm),
+            'view': InstanceOf(View)
+        })
+
+    def test_form_preview(self):
+        view = FormView.as_view(
+            form_class=ExampleForm,
+            success_url='/success/',
+            template_name='example.html'
+        )
+        response = self.get(view)
+
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.template_name, ['example.html'])
+        self.assertContext(response, {
+            'form': InstanceOf(BaseForm),
+            'view': InstanceOf(View)
+        })
+
+    def test_misconfigured_form_view_no_form_class(self):
+        # A template view with no `form_class` is improperly configured.
+        view = FormView.as_view(
+            success_url='/success/',
+            template_name='example.html'
+        )
+        self.assertRaises(ImproperlyConfigured, self.get, view)
+
+    def test_misconfigured_form_view_no_success_url(self):
+        # A template view with no `success_url` is improperly configured.
+        view = FormView.as_view(
+            form_class=ExampleForm,
+            template_name='example.html'
+        )
+        self.assertRaises(ImproperlyConfigured, self.post, view, data={'text': 'example'})
